from setuptools import setup, find_packages

setup(
    name='smartarchive',
    version='0.1',
    description='A simple Python library of ai doc manage',
    author='bjzy',
    author_email='your_email@example.com',
    packages=find_packages(),
    python_requires='>=3.11',
    install_requires=[],
)